<?php

namespace Sukellos\Admin;

use Sukellos\ScrapioAPI\WP_Scrapio_API;
use Sukellos\ScrapioAPI\WP_Scrapio_API_Gmap_Search_Param;
use Sukellos\WP_Scrapio_Importer;
use Sukellos\WP_Scrapio_Requester;
use Sukellos\WP_Sukellos_Scrapio_Loader;
use Sukellos\WPFw\Utils\WP_Log;
use Sukellos\WPFw\WP_Plugin_Admin;
use Sukellos\WPFw\AdminBuilder\Admin_Builder;
use Sukellos\WPFw\AdminBuilder\Item_Type;
use Sukellos\WPFw\Singleton;
use Exception;
use DateTimeZone;

defined( 'ABSPATH' ) or exit;

/**
 * Admin class.
 *
 * @since 1.0.0
 */
class WP_Scrapio_Admin_Import extends WP_Plugin_Admin {

    // Use Trait Singleton
    use Singleton;

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {

        parent::init();

        add_action( 'wp_ajax_sukellos_scrapio_select_activity', array( $this, 'execute_select_activity' ) );
        add_action( 'wp_ajax_sukellos_scrapio_select_locations', array( $this, 'execute_select_locations' ) );
        add_action( 'wp_ajax_sukellos_scrapio_gmap_search', array( $this, 'execute_gmap_search' ) );
    }

    /**
     *          ===============
     *      =======================
     *  ============ HOOKS ===========
     *      =======================
     *          ===============
     */

    /***
     * AJAX Execute the _select_activity
     */
    public function execute_select_activity() {

        try {

            WP_Log::debug(__METHOD__, [], 'wp-sukellos-fw');

            // Request Scrap.io
            $scrapio_types = WP_Scrapio_API::instance()->gmap_types( $_GET['q'], 'fr', false );
            $types = $scrapio_types->get_types();

            // Utiliser wp_send_json_success pour envoyer la réponse
            wp_send_json_success( array('results' => $types) );

        } catch (Exception $ex) {

            WP_Log::error(__METHOD__.' Error during product synchronization', ['code' => $ex->getCode(), 'message' => $ex->getMessage()], 'wp-sukellos-fw');
            wp_send_json_error('Error during request <br/> Code: '.$ex->getCode().' <br/> Message: '.$ex->getMessage().'<br/> See logging for more detailed informations.');
        }
        exit;
    }

    /***
     * AJAX Execute the _select_locations
     */
    public function execute_select_locations() {

        try {

            WP_Log::debug(__METHOD__, [], 'wp-sukellos-fw');

            // Request Scrap.io depending on given params
            $country_code = $_POST['country'];
            $type = $_POST['type'];
            $admin1_code = isset( $_POST['admin1'] ) ? $_POST['admin1'] : null;
            $admin2_code = isset( $_POST['admin2'] ) ? $_POST['admin2'] : null;
            $search_term = isset( $_POST['search_term'] ) ? $_POST['search_term'] : null;

            $scrapio_gmap_locations = WP_Scrapio_API::instance()->gmap_locations( $country_code, $type, $admin1_code, $admin2_code, $search_term, false );
            $locations = $scrapio_gmap_locations->get_locations();

            // Utiliser wp_send_json_success pour envoyer la réponse
            wp_send_json_success( array('results' => $locations) );

        } catch (Exception $ex) {

            WP_Log::error(__METHOD__.' Error during product synchronization', ['code' => $ex->getCode(), 'message' => $ex->getMessage()], 'wp-sukellos-fw');
            wp_send_json_error('Error during request <br/> Code: '.$ex->getCode().' <br/> Message: '.$ex->getMessage().'<br/> See logging for more detailed informations.');
        }
        exit;
    }

    /***
     * AJAX Execute the gmap_search
     */
    public function execute_gmap_search() {

        try {

            WP_Log::debug(__METHOD__, [], 'wp-sukellos-fw');

            // Request Scrap.io
            $country = $_POST['country'];
            $type = $_POST['type'];
            $admin1 = null;
            $admin2 = null;
            $city = null;
            if ( isset( $_POST['admin1'] ) ) $admin1 = $_POST['admin1'];
            if ( isset( $_POST['admin2'] ) ) $admin2 = $_POST['admin2'];
            if ( isset( $_POST['city'] ) ) $city = $_POST['city'];

            // Perform request
            $result = WP_Scrapio_Importer::instance()->schedule_gmap_search( $country, $type, $admin1, $admin2, $city );

            if ( $result === TRUE ) {

                $html_response = '<strong>'.__( 'Gmap search import from Scrap.io scheduled ', 'wp-sukellos-fw' );
            } else {

                $html_response = '<strong>'.__( 'Number of results: ', 'wp-sukellos-fw' ).'</strong>'.$result->get_count();
            }
            WP_Log::debug(__METHOD__, ['result'=>$result, 'html_response'=>$html_response ], 'wp-sukellos-fw');


            // Utiliser wp_send_json_success pour envoyer la réponse
            wp_send_json_success( $html_response );

        } catch (Exception $ex) {

            WP_Log::error(__METHOD__.' Error during product synchronization', ['code' => $ex->getCode(), 'message' => $ex->getMessage()], 'wp-sukellos-fw');
            wp_send_json_error('Error during request <br/> Code: '.$ex->getCode().' <br/> Message: '.$ex->getMessage().'<br/>');
        }
        exit;
    }

    /***
     * Adding CSS and JS into header
     * Default add assets/admin.css and assets/admin.js
     */
    public function admin_enqueue_scripts() {

        wp_enqueue_style('jsdelivr-select2-css', 'https://cdn.jsdelivr.net/npm/select2@4.1.0-rc.0/dist/css/select2.min.css');
        wp_enqueue_script('jsdelivr-select2-js', 'https://cdn.jsdelivr.net/npm/select2@4.1.0-rc.0/dist/js/select2.min.js', array('jquery'), null, true);

        // JS
        wp_enqueue_script('sukellos_scrapio_select_activity_js', WP_Sukellos_Scrapio_Loader::instance()->get_plugin_dir_url().'assets/js/select-activity.js', array('jquery'), '1.0', true);

        // Localize the script with the AJAX URL
        wp_localize_script('sukellos_scrapio_select_activity_js', 'ajax_object', array(
            'ajax_url' => admin_url('admin-ajax.php'),
        ));

    }

    /***
     * Admin page
     * Settings managed by main Sukellos Fw Tools admin
     */
    public function create_items()
    {
        // To improve code readability
        $plugin_name = WP_Sukellos_Scrapio_Loader::instance()->get_name();
        $options_suffix_param = WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param();
/*
        // Check if license restriction applied
        if ( $this->is_license_restricted() ) {

            // Admin page.
            $admin_page = Admin_Builder::instance()->create_admin_page(
                array(
                    'name' => $plugin_name.' - '.__( 'Settings', 'wp-sukellos-fw' ),
                    'title' => __( 'Settings', 'wp-sukellos-fw' ),
                    'id' => $options_suffix_param . '_settings',
                    'capability' => 'manage_options',
                    'parent_id' => $options_suffix_param.'_scrapio_admin_page',
                    'icon' => 'dashicons-smiley', // Menu icon for top level menus only https://developer.wordpress.org/resource/dashicons/#editor-paste-word
                    'position' => 27,
                    'use_tabs' => false,
                )
            );

            // Create a note
            $admin_page->create_note(
                array(
                    // Common
                    'id' => $options_suffix_param.'_note_license_invalid2',
                    'name' => __( 'ATTENTION', 'wp-sukellos-fw' ),
                    'desc' => __( 'Thank you to validate your license, in order to be able to use all the functionalities of this plugin. If you do not yet have a license key, you can purchase one <a href="'.WP_Sukellos_Scrapio_Loader::instance()->get_sales_page_url().'" target="_blank">here</a>', 'wp-sukellos-fw' ),
                    'sukellos_rule_exception' => true,
                )
            );
            return;
        }
*/
        // Admin page.
        $admin_page = Admin_Builder::instance()->create_admin_page(
            array(
                'name' => $plugin_name.' - '.__( 'Import Tool', 'wp-sukellos-fw' ),
                'title' => __( 'Import Tool', 'wp-sukellos-fw' ),
                'id' => $options_suffix_param . '_import',
                'capability' => 'manage_options',
                'parent_id' => $options_suffix_param.'_scrapio_admin_page',
                'icon' => 'dashicons-smiley', // Menu icon for top level menus only https://developer.wordpress.org/resource/dashicons/#editor-paste-word
                'position' => 22,
            )
        );

        $admin_page->create_header(
            array(
                'id' => $options_suffix_param.'_header_import',
                'name' => __( 'Import', 'wp-sukellos-fw' ),
                'desc' => __( 'Importation of Gmap entities from Scrap.io', 'wp-sukellos-fw' ),
            )
        );

        //
        // Create an select option field for activities
        //
        $timestamp = wp_next_scheduled( 'execute_gmap_search_hook' );

        // Scheduled
        if ( $timestamp !== false ) {

            $date_time_zone = new DateTimeZone( get_option('timezone_string') );
            $next_run = wp_date('Y-m-d H:i:s', $timestamp, $date_time_zone );

            $html_content = '<p>'.__( 'An import action from Scrap.io is already in progress. Please wait.', 'wp-sukellos-fw' ).'</p>';
            $html_content .= '<p>'.__( 'Next request', 'wp-sukellos-fw' ).': '.$next_run.'</p>';

        }
        // Nothing scheduled...
        else {

            $html_content = '            
            <p><select id="sukellos_scrapio_select_activity" style="width: 300px;">
                <option value="">'.__( 'Select activity', 'wp-sukellos-fw' ).'</option>
            </select></p> 
            
            <p><select id="sukellos_scrapio_select_country" style="width: 300px;">
                <option value="">'.__( 'Select country', 'wp-sukellos-fw' ).'</option>
                <option value="AL">Albania</option>
                <option value="DZ">Algeria</option>
                <option value="AD">Andorra</option>
                <option value="AO">Angola</option>
                <option value="AR">Argentina</option>
                <option value="AM">Armenia</option>
                <option value="AU">Australia</option>
                <option value="AT">Austria</option>
                <option value="AZ">Azerbaijan</option>
                <option value="BD">Bangladesh</option>
                <option value="BY">Belarus</option>
                <option value="BE">Belgium</option>
                <option value="BJ">Benin</option>
                <option value="BO">Bolivia</option>
                <option value="BW">Botswana</option>
                <option value="BR">Brazil</option>
                <option value="VG">British Virgin Islands</option>
                <option value="BG">Bulgaria</option>
                <option value="BF">Burkina Faso</option>
                <option value="KH">Cambodia</option>
                <option value="CM">Cameroon</option>
                <option value="CA">Canada</option>
                <option value="CF">Central African Republic</option>
                <option value="CL">Chile</option>
                <option value="CN">China</option>
                <option value="CO">Colombia</option>
                <option value="CG">Congo</option>
                <option value="CR">Costa Rica</option>
                <option value="HR">Croatia</option>
                <option value="CY">Cyprus</option>
                <option value="CZ">Czech Republic</option>
                <option value="DK">Denmark</option>
                <option value="DJ">Djibouti</option>
                <option value="DO">Dominican Republic</option>
                <option value="EC">Ecuador</option>
                <option value="EG">Egypt</option>
                <option value="EE">Estonia</option>
                <option value="ET">Ethiopia</option>
                <option value="FI">Finland</option>
                <option value="FR">France</option>
                <option value="GF">French Guyana</option>
                <option value="PF">French Polynesia</option>
                <option value="GA">Gabon</option>
                <option value="GE">Georgia</option>
                <option value="DE">Germany</option>
                <option value="GH">Ghana</option>
                <option value="GR">Greece</option>
                <option value="GP">Guadeloupe</option>
                <option value="GT">Guatemala</option>
                <option value="GN">Guinea</option>
                <option value="HT">Haiti</option>
                <option value="HK">Hong Kong</option>
                <option value="HU">Hungary</option>
                <option value="IS">Iceland</option>
                <option value="IN">India</option>
                <option value="ID">Indonesia</option>
                <option value="IE">Ireland</option>
                <option value="RE">Island of Reunion</option>
                <option value="IL">Israel</option>
                <option value="IT">Italy</option>
                <option value="CI">Ivory Coast</option>
                <option value="JP">Japan</option>
                <option value="JO">Jordan</option>
                <option value="KE">Kenya</option>
                <option value="XK">Kosovo</option>
                <option value="LA">Laos</option>
                <option value="LV">Latvia</option>
                <option value="LB">Lebanon</option>
                <option value="LR">Liberia</option>
                <option value="LI">Liechtenstein</option>
                <option value="LT">Lithuania</option>
                <option value="LU">Luxembourg</option>
                <option value="MG">Madagascar</option>
                <option value="MW">Malawi</option>
                <option value="MY">Malaysia</option>
                <option value="MV">Maldives</option>
                <option value="ML">Mali</option>
                <option value="MT">Malta</option>
                <option value="MQ">Martinique</option>
                <option value="MU">Mauritius</option>
                <option value="YT">Mayotte</option>
                <option value="MX">Mexico</option>
                <option value="MD">Moldova</option>
                <option value="MC">Monaco</option>
                <option value="ME">Montenegro</option>
                <option value="MA">Morocco</option>
                <option value="MZ">Mozambique</option>
                <option value="MM">Myanmar</option>
                <option value="NA">Namibia</option>
                <option value="NL">Netherlands</option>
                <option value="NC">New Caledonia</option>
                <option value="NZ">New Zealand</option>
                <option value="NE">Niger</option>
                <option value="NG">Nigeria</option>
                <option value="NO">Norway</option>
                <option value="OM">Oman</option>
                <option value="PK">Pakistan</option>
                <option value="PA">Panama</option>
                <option value="PY">Paraguay</option>
                <option value="PE">Peru</option>
                <option value="PH">Philippines</option>
                <option value="PL">Poland</option>
                <option value="PT">Portugal</option>
                <option value="PR">Puerto Rico</option>
                <option value="QA">Qatar</option>
                <option value="RO">Romania</option>
                <option value="RU">Russia</option>
                <option value="BL">Saint Barthelemy</option>
                <option value="SV">Salvador</option>
                <option value="SA">Saudi Arabia</option>
                <option value="SN">Senegal</option>
                <option value="RS">Serbia</option>
                <option value="SC">Seychelles</option>
                <option value="SG">Singapore</option>
                <option value="SK">Slovakia</option>
                <option value="SI">Slovenia</option>
                <option value="SO">Somalia</option>
                <option value="ZA">South Africa</option>
                <option value="KR">South Korea</option>
                <option value="ES">Spain</option>
                <option value="LK">Sri Lanka</option>
                <option value="SE">Sweden</option>
                <option value="CH">Switzerland</option>
                <option value="TW">Taiwan</option>
                <option value="TZ">Tanzania</option>
                <option value="TH">Thailand</option>
                <option value="TG">Togo</option>
                <option value="TN">Tunisia</option>
                <option value="TR">Turkey</option>
                <option value="UG">Uganda</option>
                <option value="UA">Ukraine</option>
                <option value="AE">United Arab Emirates</option>
                <option value="GB">United Kingdom</option>
                <option value="US">United States</option>
                <option value="UY">Uruguay</option>
                <option value="VE">Venezuela</option>
                <option value="VN">Vietnam</option>
                <option value="WF">Wallis and Futuna</option>
            </select></p> 
            
            <p><select id="sukellos_scrapio_select_level1_division" style="width: 300px;">
                <option value="">'.__( 'Select level 1 division', 'wp-sukellos-fw' ).'</option>
            </select></p> 
            
            <p><select id="sukellos_scrapio_select_level2_division" style="width: 300px;">
                <option value="">'.__( 'Select level 2 division', 'wp-sukellos-fw' ).'</option>
            </select></p> 
            
            <p><select id="sukellos_scrapio_select_city" style="width: 300px;">
                <option value="">'.__( 'Select city', 'wp-sukellos-fw' ).'</option>
            </select></p> 
            
            <p>'.__( 'Please note, each import consumes Scrap.io credits', 'wp-sukellos-fw' ).'</p>
            <p>
                <button id="sukellos_scrapio_gmap_search" name="action" value="sukellos_scrapio_gmap_search" class="button button-primary">
                        '.__( 'Import from Scrap.io', 'wp-sukellos-fw' ).'
                </button>
            </p>
            ';
        }
        $admin_page->create_content(
            array(
                'id' => $options_suffix_param . '_select_activity_select',
                'name' => __( 'Select activity select', 'wp-sukellos-fw' ),
                'content' => $html_content,
            )
        );
        $admin_page->create_content(
            array(
                'id' => 'sukellos_scrapio_import_result',
                'name' => '',
                'content' => '',
            )
        );
    }

}