<?php

namespace Sukellos\Admin;

use Sukellos\ScrapioAPI\WP_Scrapio_API;
use Sukellos\WP_Sukellos_Scrapio_Loader;
use Sukellos\WPFw\AdminBuilder\Admin_Builder;
use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\WP_Plugin_Admin;
use Sukellos\WPFw\Utils\WP_Log;
use Sukellos\WPFw\AdminBuilder\Item_Type;
use Exception;

defined( 'ABSPATH' ) or exit;

/**
 * Admin class.
 * Main admin is used as controller to init admin menu and all other admin pages
 *
 * @since 1.0.0
 */
class WP_Sukellos_Scrapio_Admin extends WP_Plugin_Admin {

    // Use Trait Singleton
    use Singleton;

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {

        parent::init();
    }


    /**
     * Get sub admin instances.
     * Must be used ONLY in main admin, to init sub admin instances correctly
     *
     * @return array of WP_Plugin_Admin instances
     */
    public function get_plugin_sub_admins() {

        if ( !is_user_logged_in() || !is_admin() ) {

            return array();
        }
        
        // Others pages only appears if Scrap.io connected
        $connected = $this->is_scrapio_connected();
        $sub_admins = array();

        if ( $connected !== false ) {

            $sub_admins = array(
                WP_Scrapio_Admin_Settings::instance(),
                WP_Scrapio_Admin_Import::instance(),
                WP_Scrapio_Admin_Searches::instance(),
                WP_Scrapio_Admin_Entities_List::instance(),
                WP_Scrapio_Admin_Entity::instance(),
            );

            // Activate debug mode ?
            $enable_debug = Admin_builder::get_option(WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_enable_debug', Item_Type::ENABLE);
            if ( $enable_debug ) {

                $sub_admins[] = WP_Scrapio_Admin_Debug::instance();
            }
        }

        return $sub_admins;
    }

    /**
     * Gets the plugin configuration URL
     * This is used to build actions list in plugins page
     * Leave blank ('') to disable
     *
     * @since 1.0.0
     *
     * @return string plugin settings URL
     */
    public function get_settings_url() {

        return admin_url( 'admin.php?page='.WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_scrapio_admin_page' );
    }

    /**
     *          ===============
     *      =======================
     *  ============ HOOKS ===========
     *      =======================
     *          ===============
     */

    /***
     * Adding CSS and JS into header
     * Default add assets/admin.css and assets/admin.js
     */
    public function admin_enqueue_scripts() {

        // CSS
        wp_enqueue_style('sukellos_scrapio_admin_css', WP_Sukellos_Scrapio_Loader::instance()->get_plugin_dir_url().'assets/css/admin.css', array(), '1.0', 'all');

        // JS
        wp_enqueue_script('sukellos_scrapio_admin_js', WP_Sukellos_Scrapio_Loader::instance()->get_plugin_dir_url().'assets/js/admin.js', array('jquery'), '1.0', true);
    }

    /**
     * Check if user has subscribed and is connected to Scrap.io
     * @return void false if not connected, else the subscription response
     */
    private function is_scrapio_connected() {

        $subscription = false;
        try {

            // Request Scrap.io
            $subscription = WP_Scrapio_API::instance()->subscription( false );

        } catch (Exception $ex) {
        }
        return $subscription;
    }


    /***
     * Admin page
     * Settings managed by main Sukellos Fw Tools admin
     */
    public function create_items() {

        // To improve code readability
        $plugin_name = WP_Sukellos_Scrapio_Loader::instance()->get_name();
        $options_suffix_param = WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param();

        // Admin page.
        $admin_page = Admin_Builder::instance()->create_admin_page(
            array(
                'name' => $plugin_name,
                'title' => $plugin_name,
                'id' => $options_suffix_param . '_scrapio_admin_page',
                'capability' => 'manage_options',
                'parent_id' => null,
                'icon' => 'dashicons-smiley', // Menu icon for top level menus only https://developer.wordpress.org/resource/dashicons/#editor-paste-word
                'position' => 11,
                'desc' => __( 'Import data from Scrap.io', 'wp-sukellos-fw' ),
                'use_reset' => true,
                'save' => __('Save', 'wp-sukellos-fw'), // Text for save button
                'reset' => __('Reset', 'wp-sukellos-fw'), // Text for reset button
                'reset_question' => __('Do you really want to reset form?', 'wp-sukellos-fw' ), // Question asked to confirm reset
                'use_tabs' => false,
            )
        );

        global $pagenow;
        if ( $pagenow != 'admin.php' || ( isset($_GET['page']) && $_GET['page'] != $options_suffix_param . '_scrapio_admin_page' ) ) {
            return;
        }

        $subscription = $this->is_scrapio_connected();
        $html_content = '
            <div class="scrapio-logo-container">
                <img src="'.WP_Sukellos_Scrapio_Loader::instance()->get_plugin_dir_url().'/assets/img/logo-scrapio-262-69.png" alt="Logo Scrap.io">
            </div>
            ';
        if ( $subscription !== false ) {

            $html_content .= '
                <div class="grid">
                    <div class="card">
                        <h2>'.__( 'Connection status', 'wp-sukellos-fw' ).'</h2>
                        <p><strong>'.__( 'Connected:', 'wp-sukellos-fw' ).'</strong> '.__( 'Yes', 'wp-sukellos-fw' ).'</p>
                        <p><strong>'.__( 'Plan:', 'wp-sukellos-fw' ).'</strong> '.$subscription->get_plan().'</p>
                        <p><strong>'.__( 'Active:', 'wp-sukellos-fw' ).'</strong> '.( $subscription->is_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' ) ).'</p>
                        <p><strong>'.__( 'On trial:', 'wp-sukellos-fw' ).'</strong> '.( $subscription->is_on_trial() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' ) ).'</p>
                        <p><strong>'.__( 'Pending cancellation:', 'wp-sukellos-fw' ).'</strong> '.( $subscription->is_pending_cancelation() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' ) ).'</p>
                        <p><strong>'.__( 'Renewal date:', 'wp-sukellos-fw' ).'</strong> '.$subscription->get_renewal_date().'</p>
                    </div>
            
                    <div class="card features">
                        <h2>'.__( 'Available features', 'wp-sukellos-fw' ).'</h2>
                        <ul>
                            <li class="'.( $subscription->is_search_city_active() ? 'ok' : 'nok').'"><span>'.__( 'Search by city:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_search_city_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                            <li class="'.( $subscription->is_search_admin2_code_active() ? 'ok' : 'nok').'"><span>'.__( 'Search by level 2 division:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_search_admin2_code_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                            <li class="'.( $subscription->is_search_admin1_code_active() ? 'ok' : 'nok').'"><span>'.__( 'Search by level 1 division:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_search_admin1_code_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                            <li class="'.( $subscription->is_search_whole_country_active() ? 'ok' : 'nok').'"><span>'.__( 'Search on whole country:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_search_whole_country_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                            <li class="'.( $subscription->is_essential_search_filters_active() ? 'ok' : 'nok').'"><span>'.__( 'Essential search filters:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_essential_search_filters_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                            <li class="'.( $subscription->is_advanced_search_filters_active() ? 'ok' : 'nok').'"><span>'.__( 'Advanced search filters:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_advanced_search_filters_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                            <li class="'.( $subscription->is_gmap_export_additional_fields_active() ? 'ok' : 'nok').'"><span>'.__( 'GMap additional fields in exports:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_gmap_export_additional_fields_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                            <li class="'.( $subscription->is_web_export_additional_fields_active() ? 'ok' : 'nok').'"><span>'.__( 'Website additional fields in exports:', 'wp-sukellos-fw' ).'</span> '.( $subscription->is_web_export_additional_fields_active() ? __( 'Yes', 'wp-sukellos-fw' ) : __( 'No', 'wp-sukellos-fw' )).'</li>
                        </ul>
                    </div>
                </div>
            
                <div class="credits">
                    <div class="credit-block">
                        <h2>'.__( 'Consumed credits', 'wp-sukellos-fw' ).'</h2>
                        <p>'.$subscription->get_export_credits_consumed().'</p>
                    </div>
                    <div class="credit-block">
                        <h2>'.__( 'Remaining credits', 'wp-sukellos-fw' ).'</h2>
                        <p>'.$subscription->get_export_credits_remaining().'</p>
                    </div>
                    <div class="credit-block">
                        <h2>'.__( 'Total credits', 'wp-sukellos-fw' ).'</h2>
                        <p>'.$subscription->get_export_credits_total().'</p>
                    </div>
                </div>
            ';
        }
        else {

            $html_content .= '
                <div class="scrapio-register-container">
                
                    <h1>Connectez votre plugin à Scrap.io</h1>
                    <p>Votre plugin n\'est pas encore connecté au service Scrap.io. Pour utiliser toutes les fonctionnalités, veuillez connecter votre compte.</p>
                    <p><strong>Pour vous inscrire, suivez ce lien :</strong></p>
                    <a href="https://scrap.id/s/ppEP" target="_blank" class="button button-primary">S\'inscrire sur Scrap.io</a>
                </div>
            ';
        }

        $admin_page->create_content(
            array(
                'id' => $options_suffix_param . '_subscription_infos',
                'name' => __( 'Subscription infos', 'wp-sukellos-fw' ),
                'content' => $html_content,
            )
        );


        $admin_page->create_header(
            array(
                'id' => $options_suffix_param.'_header_scrapio',
                'name' => __( 'Scrap.io setup', 'wp-sukellos-fw' ),
                'desc' => __( 'Configuring the connection with the Srcap.io server', 'wp-sukellos-fw' ),
            )
        );

        $admin_page->create_option(
            array(
                'type' => Item_type::TEXT,
                'id'   => $options_suffix_param.'_scrapio_endpoint_url',
                'name' => __( 'Scrap.io base url for all endpoints. Do no forget end slash.', 'wp-sukellos-fw' ),
                'desc' => '',
                'default' => 'https://scrap.io/api/v1/',
            )
        );

        $admin_page->create_option(
            array(
                'type' => Item_type::TEXT,
                'id'   => $options_suffix_param.'_scrapio_key',
                'name' => __( 'Scrap.io API key', 'wp-sukellos-fw' ),
                'desc' => '',
            )
        );
    }
}