<?php

namespace Sukellos;

use Sukellos\Dao\WP_Scrapio_Gmap_Search_Dao;
use Sukellos\ScrapioAPI\WP_Scrapio_API_Gmap_Search_Param;
use Sukellos\WPFw\AdminBuilder\Admin_Builder;
use Sukellos\WPFw\AdminBuilder\Item_Type;
use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\Utils\WP_Log;

defined( 'ABSPATH' ) or exit;

/**
 * Scrap.io requester
 *
 * @since 1.0.0
 */
class WP_Scrapio_Cacher {

    // Use Trait Singleton
    use Singleton;

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {
    }

    /**
     * Abort gmap search if cache activated and already requested
     * @param $args
     * @return boolean true if gmap search can be done, otherwise false
     */
    public function gmap_search_in_cache( $args ) {

        // The algo is the following :
        // Enable/disable cache mode to avoid repeating identical or more specific requests.
        // For example, if cache mode is enabled and a request has already been made for FR/Bretagne,
        // the request for FR/Bretagne will not be replayed,
        // nor will the more specific request FR/Bretagne/Finistère, since the results for Finistère are already stored.
        // This helps to save Scrap.io credits.
        $enable_caching = Admin_Builder::get_option(WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_enable_caching', Item_type::ENABLE);
        if ( !$enable_caching ) return false;

        // If a request exist with :
        // - same type
        // - same country_code
        // - exact or more wide params admin1, admin2, city
        // - exactly the same other args
        // Then no need to request again
        $type = $args[WP_Scrapio_API_Gmap_Search_Param::TYPE];
        $country_code = $args[WP_Scrapio_API_Gmap_Search_Param::COUNTRY_CODE];
        $admin1 = null;
        $admin2 = null;
        $city = null;
        if ( array_key_exists( WP_Scrapio_API_Gmap_Search_Param::ADMIN1_CODE, $args ) ) $admin1 = $args[WP_Scrapio_API_Gmap_Search_Param::ADMIN1_CODE];
        if ( array_key_exists( WP_Scrapio_API_Gmap_Search_Param::ADMIN2_CODE, $args ) ) $admin2 = $args[WP_Scrapio_API_Gmap_Search_Param::ADMIN2_CODE];
        if ( array_key_exists( WP_Scrapio_API_Gmap_Search_Param::CITY, $args ) ) $city = $args[WP_Scrapio_API_Gmap_Search_Param::CITY];
        $results = WP_Scrapio_Gmap_Search_Dao::instance()->retrieve_gmap_search( $type, $country_code, $admin1, $admin2, $city );

        if ( empty( $results ) ) {

            WP_Log::debug( __METHOD__, ['$results'=>$results], 'wp-sukellos-fw' );
            return false;
        }

        // Args to be compared
        $key_to_compare = array(
            WP_Scrapio_API_Gmap_Search_Param::GMAP_IS_MAIN_TYPE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_IS_CLOSED,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_HAS_WEBSITE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_HAS_PHONE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_PRICE_RANGE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_COUNT_LTE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_COUNT_LT,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_COUNT_GTE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_COUNT_GT,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_RATING_LTE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_RATING_LT,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_RATING_GTE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_REVIEWS_RATING_GT,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_PHOTOS_COUNT_LTE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_PHOTOS_COUNT_LT,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_PHOTOS_COUNT_GTE,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_PHOTOS_COUNT_GT,
            WP_Scrapio_API_Gmap_Search_Param::GMAP_IS_CLAIMED,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_TITLE,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_META_KEYWORDS,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_META_DESCRIPTION,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_CONTACT_PAGES,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_EMAILS,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_PHONES,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_FACEBOOK,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_YOUTUBE,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_TWITTER,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_INSTAGRAM,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_LINKEDIN,
            WP_Scrapio_API_Gmap_Search_Param::WEBSITE_HAS_AD_PIXELS,
        );

        // Check other params
        foreach ( $results as $result ) {

            $old_args = json_decode( $result['arguments'], true );
            WP_Log::debug( __METHOD__, ['new search args'=>$args, 'args from old search'=>$old_args], 'wp-sukellos-fw' );

            $diff_old_new = array_diff_assoc( $old_args, $args );
            $diff_new_old = array_diff_assoc( $args, $old_args );
            $diff = array_merge($diff_old_new, $diff_new_old);
            WP_Log::debug( __METHOD__, ['$diff'=>$diff], 'wp-sukellos-fw' );

            // If the diff is one of checked args, then this is not exactly the same request
            foreach ( $diff as $key => $value ) {

                if ( in_array( $key, $key_to_compare ) ) continue 2;
            }
            // No diff found, then in cache
            return true;
        }

        return false;
    }
}
