<?php

namespace Sukellos;

use Sukellos\Dao\WP_Scrapio_Gmap_Entities_Dao;
use Sukellos\WPFw\AdminBuilder\Admin_Builder;
use Sukellos\WPFw\AdminBuilder\Item_Type;
use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\Utils\WP_Log;

defined( 'ABSPATH' ) or exit;

/**
 * Scrap.io CPT mapper will created a CPT per Gmap entity
 *
 * @since 1.0.0
 */
class WP_Scrapio_CPT_Mapper {

    // Use Trait Singleton
    use Singleton;

    // Supported field ids
    const SUPPORTED_FIELD_KEYS = array(
        'Entity_Id',
        'Place_Id',
        'Link',
        'Phone',
        'Photos',
        'Website',
        'Google_Id',
        'Reviews_Id',
        'Price_Range',
        'Emails',
        'Twitter',
        'Youtube',
        'Facebook',
        'Linkedin',
        'Instagram',
        'Contact_Pages',
        'Location_City',
        'Reviews_Count',
        'Location',
        'Location_Street_1',
        'Location_Street_2',
        'Reviews_Per_Score',
        'Phone_International',
        'Location_Admin1_Code',
        'Location_Admin2_Code',
        'Location_Postal_Code',
        'Location_Country_Code',
        'Location_Full_Address',
    );
    const SCRAPIO_ACF_GROUP_ID = 'group_scrapio_gmap_entity';
    const WORKING_HOURS_FIELD_KEYS = 'Working_Hours';

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {

        $enable_acf_field_keys = Admin_builder::get_option(WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_enable_acf_field_keys', Item_Type::ENABLE);
        if ( $enable_acf_field_keys ) add_action( 'acf/include_fields', array( $this, 'action_acf_include_fields' ), 10 );

        $enable_acf_working_hours_field_key = Admin_builder::get_option(WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_enable_acf_working_hours_field_key', Item_Type::ENABLE);
        if ( $enable_acf_working_hours_field_key ) add_action( 'acf/include_fields', array( $this, 'action_acf_include_working_hours_fields' ), 10 );
    }

    /**
     * Get supported field keys, authorized in annuaire settings
     * @return array|false|mixed
     */
    public function get_supported_field_keys() {

        // Get from settings
        $supported_field_keys = Admin_Builder::get_option( WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_supported_field_keys', Item_Type::MULTICHECK );
        if ( !is_array( $supported_field_keys ) ) $supported_field_keys = (array) $supported_field_keys;
        WP_Log::debug( __METHOD__, ['$supported_field_keys'=>$supported_field_keys], 'wp-sukellos-fw' );

        return $supported_field_keys;
    }

    /**
     * Get supported ACF fields instances, authorized in annuaire settings
     * @return array|false|mixed
     */
    public function get_supported_acf_field_instances() {

        // Get from settings
        $supported_field_keys = $this->get_supported_field_keys();

        // All fields are taken from Scrap.io gmap search
        $fields = array();
        foreach ( $supported_field_keys as $field_key ) {

            $class_name = "Sukellos\\Scrapio\\".$field_key;
            $field_instance = new $class_name();
            $fields[ $field_key ] = $field_instance;
        }
        return $fields;
    }

    /**
     * Generate ACF fields for Gmap entities
     * @return void
     */
    public function action_acf_include_fields() {

        // Check if CPT has been chosen
        $gmap_entities_cpt_type = $this->get_gmap_entity_cpt_type();
        WP_Log::debug( __METHOD__, ['$gmap_entities_cpt_type'=>$gmap_entities_cpt_type], 'wp-sukellos-fw' );

        if ( is_null( $gmap_entities_cpt_type ) ) return;

        if ( ! function_exists( 'acf_add_local_field_group' ) ) {
            return;
        }

        // Get from settings
        $supported_field_keys = $this->get_supported_field_keys();

        // All fields are taken from Scrap.io gmap search
        $fields = array();
        foreach ( $supported_field_keys as $field_key ) {

            $class_name = "Sukellos\\Scrapio\\".$field_key;
            $field_instance = new $class_name();
            $fields[] = $field_instance->get_acf_field_definition();
            WP_Log::debug( __METHOD__, ['ACF field definition'=>$field_instance->get_acf_field_definition()], 'wp-sukellos-fw' );
        }

        // Define fields group
        $group = array(

            /* (string) Unique identifier for field group. Must begin with 'group_' */
            'key' => self::SCRAPIO_ACF_GROUP_ID,

            /* (string) Visible in metabox handle */
            'title' => __('Scrap.io - Gmap fields', 'wp-sukellos-fw' ),

            /* (array) An array of fields */
            'fields' => $fields,

            /* (array) An array containing 'rule groups' where each 'rule group' is an array containing 'rules'.
            Each group is considered an 'or', and each rule is considered an 'and'. */
            'location' => array(
                array(
                    array(
                        'param' => 'post_type',
                        'operator' => '==',
                        'value' => $gmap_entities_cpt_type,
                    ),
                ),
            ),

            /* (int) Field groups are shown in order from lowest to highest. Defaults to 0 */
            'menu_order' => 0,

            /* (string) Determines the position on the edit screen. Defaults to normal. Choices of 'acf_after_title', 'normal' or 'side' */
            'position' => 'normal',

            /* (string) Determines the metabox style. Defaults to 'default'. Choices of 'default' or 'seamless' */
            'style' => 'default',

            /* (string) Determines where field labels are places in relation to fields. Defaults to 'top'.
            Choices of 'top' (Above fields) or 'left' (Beside fields) */
            'label_placement' => 'top',

            /* (string) Determines where field instructions are places in relation to fields. Defaults to 'label'.
            Choices of 'label' (Below labels) or 'field' (Below fields) */
            'instruction_placement' => 'label',

            /* (array) An array of elements to hide on the screen */
            'hide_on_screen' => '',

            'active' => true,
            'description' => '',
            'show_in_rest' => 0,
        );

        // Add group
        acf_add_local_field_group( $group );
    }

    /**
     * Generate ACF fields for Gmap Working Hour entity
     * @return void
     */
    public function action_acf_include_working_hours_fields() {

        // Check if CPT has been chosen
        $gmap_entities_cpt_type = $this->get_gmap_entity_cpt_type();
        WP_Log::debug( __METHOD__, ['$gmap_entities_cpt_type'=>$gmap_entities_cpt_type], 'wp-sukellos-fw' );

        if ( is_null( $gmap_entities_cpt_type ) ) return;

        if ( ! function_exists( 'acf_add_local_field_group' ) ) {
            return;
        }

        // All fields are taken from Scrap.io gmap search
        $working_hours = new Working_Hours();
        $fields = $working_hours->get_acf_field_definition();
        WP_Log::debug( __METHOD__, ['$fields'=>$fields], 'wp-sukellos-fw' );

        // Define fields group
        $group = array(

            /* (string) Unique identifier for field group. Must begin with 'group_' */
            'key' => 'group_scrapio_gmap_entity_working_hours',

            /* (string) Visible in metabox handle */
            'title' => __('Scrap.io - Working hours', 'wp-sukellos-fw' ),

            /* (array) An array of fields */
            'fields' => $fields,

            /* (array) An array containing 'rule groups' where each 'rule group' is an array containing 'rules'.
            Each group is considered an 'or', and each rule is considered an 'and'. */
            'location' => array(
                array(
                    array(
                        'param' => 'post_type',
                        'operator' => '==',
                        'value' => $gmap_entities_cpt_type,
                    ),
                ),
            ),

            /* (int) Field groups are shown in order from lowest to highest. Defaults to 0 */
            'menu_order' => 0,

            /* (string) Determines the position on the edit screen. Defaults to normal. Choices of 'acf_after_title', 'normal' or 'side' */
            'position' => 'normal',

            /* (string) Determines the metabox style. Defaults to 'default'. Choices of 'default' or 'seamless' */
            'style' => 'default',

            /* (string) Determines where field labels are places in relation to fields. Defaults to 'top'.
            Choices of 'top' (Above fields) or 'left' (Beside fields) */
            'label_placement' => 'top',

            /* (string) Determines where field instructions are places in relation to fields. Defaults to 'label'.
            Choices of 'label' (Below labels) or 'field' (Below fields) */
            'instruction_placement' => 'label',

            /* (array) An array of elements to hide on the screen */
            'hide_on_screen' => '',

            'active' => true,
            'description' => '',
            'show_in_rest' => 0,
        );

        // Add group
        acf_add_local_field_group( $group );
    }

    /**
     * Get configured WordPress CPT type used for gmap entities
     * @return mixed|null
     */
    public function get_gmap_entity_cpt_type() {

        // Check if CPT has been chosen
        $gmap_entities_cpt_types = Admin_builder::get_option(WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_gmap_entities_cpt_type', Item_Type::SELECT);
        WP_Log::debug( __METHOD__, ['$gmap_entities_cpt_types'=>$gmap_entities_cpt_types], 'wp-sukellos-fw' );

        if ( empty( $gmap_entities_cpt_types ) ) return null;
        return $gmap_entities_cpt_types[0];
    }

    /**
     * Simply create an Entity CPT from a gmap entity
     *
     * @param $entity_id
     * @return void
     */
    private function create_cpt_from_gmap_entity( $gmap_entity ) {

        if ( is_null( $gmap_entity ) ) {

            WP_Log::warning( __METHOD__.' - No Gmap entities found for this place id', [], 'wp-sukellos-fw' );
            return false;
        }

        // Check if CPT has been chosen
        $gmap_entities_cpt_type = $this->get_gmap_entity_cpt_type();
        WP_Log::debug( __METHOD__, ['$gmap_entities_cpt_type'=>$gmap_entities_cpt_type], 'wp-sukellos-fw' );

        if ( is_null( $gmap_entities_cpt_type ) ) return false;

        // Get JSON data
        if ( !isset( $gmap_entity['entity_json'] ) || empty( $gmap_entity['entity_json'] ) ) {

            WP_Log::warning( __METHOD__.' - Cannot import empty JSON data', [], 'wp-sukellos-fw' );
        }
        $entity_json = json_decode( $gmap_entity['entity_json'], true );
        WP_Log::debug( __METHOD__, ['$entity_json'=>$entity_json ], 'wp-sukellos-fw' );

        // Search for existing entity is made on Place_Id to ensure compatibility with Sukellos Google Annuaire
        $place_id = $entity_json['place_id'];

        // Try and get actual WP entity CPT
        $wp_entity_cpt = $this->get_entity_cpt( $place_id );

        // If exists, update it
        $wp_entity_cpt_id = null;
        if ( !is_null( $wp_entity_cpt ) ) {

            $postarr = array(
                'ID' => $wp_entity_cpt->ID,
                'post_title' => $wp_entity_cpt->post_title,
                'post_name' => $wp_entity_cpt->post_name,
            );
            if ( empty( $wp_entity_cpt->post_content ) && isset( $entity_json['descriptions'] ) && !empty( $entity_json['descriptions'] ) ) {

                $postarr['post_content'] = $entity_json['descriptions'][0];
            }
            $wp_entity_cpt_id = wp_update_post( $postarr );

        }
        // Else create a new one
        else {

            $postarr = array(
                'ID' => 0, // (int) The post ID. If equal to something other than 0, the post with that ID will be updated. Default 0.
                'post_title' => $entity_json['name'], // (string) The post title. Default empty.
                'post_status' => 'publish', // (string) The post status. Default 'draft'.
                'post_type' => $gmap_entities_cpt_type, // (string) The post type. Default 'post'.
                'comment_status' => 'closed', // (string) Whether the post can accept comments. Accepts 'open' or 'closed'. Default is the value of 'default_comment_status' option.
                'ping_status' => 'closed', // (string) Whether the post can accept pings. Accepts 'open' or 'closed'. Default is the value of 'default_ping_status' option.
                'post_password' => '', // (string) The password to access the post. Default empty.
                'post_name' => $entity_json['name'], // (string) The post name. Default is the sanitized post title when creating a new post.
            );
            if ( isset( $entity_json['descriptions'] ) && !empty( $entity_json['descriptions'] ) ) {

                $postarr['post_content'] = $entity_json['descriptions'][0];
            }
            $wp_entity_cpt_id = wp_insert_post( $postarr );
        }


        if ( is_null($wp_entity_cpt_id) || is_a($wp_entity_cpt_id, 'WP_Error') ) {

            WP_Log::error( __METHOD__, ['$wp_entity_cpt_id'=>$wp_entity_cpt_id], 'wp-sukellos-fw');
            return false;
        }

        // Update ACF fields
        // Get from settings
        $supported_field_keys = Admin_Builder::get_option( WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_supported_field_keys', Item_Type::MULTICHECK );
        if ( !is_array( $supported_field_keys ) ) $supported_field_keys = (array) $supported_field_keys;
        WP_Log::debug( __METHOD__, ['$supported_field_keys'=>$supported_field_keys], 'wp-sukellos-fw' );

        // All fields are taken from Scrap.io gmap search
        foreach ( $supported_field_keys as $field_key ) {

            $class_name = "Sukellos\\Scrapio\\".$field_key;
            $field_instance = new $class_name();
            // Specific case
            if ( $field_key == 'Entity_Id' ) {

                $field_instance->set_data( $gmap_entity );
            }
            else {

                $field_instance->set_data( $entity_json );
            }
            $field_instance->update_acf_field( $wp_entity_cpt_id );
        }

        // Specific case for Working hours
        $enable_acf_working_hours_field_key = Admin_builder::get_option(WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_enable_acf_working_hours_field_key', Item_Type::ENABLE);
        if ( $enable_acf_working_hours_field_key ) {

            $working_hours = new Working_Hours();
            $working_hours->set_data( $entity_json );
            $working_hours->update_acf_field( $wp_entity_cpt_id );
        }
    }

    /**
     * Simply create an Entity CPT from a gmap entity
     *
     * @param $entity_id
     * @return void
     */
    public function create_cpt_from_gmap_entity_id( $entity_id ) {

        // First get detailed
        $gmap_entity = WP_Scrapio_Gmap_Entities_Dao::instance()->get_gmap_entity( $entity_id );
        WP_Log::debug( __METHOD__, ['$gmap_entity'=>$gmap_entity ], 'wp-sukellos-fw' );

        $this->create_cpt_from_gmap_entity( $gmap_entity );
    }

    /**
     * Simply create an Entity CPT from a gmap entity
     *
     * @param $entity_id
     * @return void
     */
    public function create_cpt_from_gmap_place_id( $place_id ) {

        // First get detailed
        $gmap_entity = WP_Scrapio_Gmap_Entities_Dao::instance()->get_gmap_entity_from_place_id( $place_id );
        WP_Log::notice( __METHOD__, ['$gmap_entity'=>$gmap_entity ], 'wp-sukellos-fw' );

        $this->create_cpt_from_gmap_entity( $gmap_entity );
    }


    /**
     * Retrieve an WP entity CPT using its entity_id from gmap search
     * @return null if no entity CPT found, else the WP_Post
     */
    public function get_entity_cpt( $gmap_place_id ) {

        // Check if CPT has been chosen
        $gmap_entities_cpt_type = $this->get_gmap_entity_cpt_type();
        WP_Log::debug( __METHOD__, ['$gmap_entities_cpt_type'=>$gmap_entities_cpt_type], 'wp-sukellos-fw' );

        if ( is_null( $gmap_entities_cpt_type ) ) return;

        // Get entity CPT infos
        $place_id = new Place_Id();
        $posts = get_posts(
            array(
                'numberposts'   => 1,
                'post_type'     => $gmap_entities_cpt_type,
                'meta_key'      => $place_id->get_acf_field_name(),
                'meta_value'    => $gmap_place_id
            )
        );
        if ( empty( $posts ) ) return null;
        $post = $posts[0];
        WP_Log::debug( __METHOD__, ['$post'=>$post], 'wp-sukellos-fw' );

        return $post;
    }
}
