<?php

namespace Sukellos;

use Sukellos\Dao\WP_Scrapio_Gmap_Entities_Dao;
use Sukellos\WPFw\Utils\WP_Log;
use WP_List_Table;

defined( 'ABSPATH' ) or exit;

/**
 * Scrap.io entities list table manager
 *
 * https://developer.wordpress.org/reference/classes/wp_list_table/
 *
 * @since 1.0.0
 */
class WP_Scrapio_Entities_List_Table extends WP_List_Table {

    private $result_message = null;

    public function __construct() {

        parent::__construct( array(
            'singular' => 'scrapio_entity',
            'plural' => 'scrapio_entities',
            'ajax' => false
        ) );
    }

    /**
     * May display a result message due to an action... if it does exist
     * @return void
     */
    public function display_result_message() {

        if ( !is_null( $this->result_message ) ) {

            echo '<div class="updated"><p>'.$this->result_message.'</p></div>';
        }
    }

    /**
     * Gets a list of columns.
     *
     * This method should be overridden to return an associative array of columns.
     * The associative array should follow the format 'slug' => array( 'Title', true ).
     * Typically, you will use this in the prepare_items() method to build part of the _column_headers property.
     *
     * @return string[]
     */
    public function get_columns() {

        $columns = array(
            'cb' => '<input type="checkbox" />',
            'name' => 'Name',
            'type' => 'Type',
            'place_id' => 'Place Id',
            'zip' => 'Zip',
            'city' => 'City',
            'actions' => 'Actions',
        );
        return $columns;
    }

    /**
     * Used to return the column value, from DB structure retrieved from gmap_entities table
     *
     * This is method that is used to render a column when no other specific method exists for that column.
     * When WP_List_Tables attempts to render your columns (within single_row_columns()), it first checks for a column-specific method. If none exists, it defaults to this method instead.
     * This method accepts two arguments, a single $item array and the $column_name (as a slug).
     * @param $item
     * @param $column_name
     * @return bool|mixed|string|void
     */
    public function column_default( $item, $column_name ) {

        switch ( $column_name ) {
            case 'name':
                // Build link to detailed entity page
                $admin_entity_url = get_site_url().'/wp-admin/admin.php?page='.WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_entity&entity_id='.$item[ 'entity_id' ];
                return '<a href="'.$admin_entity_url.'">'.$item[ $column_name ].'</a>';
            case 'type':
            case 'place_id':
            case 'zip':
            case 'city':
                return $item[ $column_name ];
            case 'actions':
                // Generates the required HTML for a list of row action links.
                // Call this method (usually from one of your column methods) to insert a row actions div.
                // The $actions parameter should be an associative array, where the key is the name of the action and the value is a link.
                $actions = array(
                    'import' => sprintf( '<a href="?page=%s&action=%s&scrapio_entity=%s">%s</a>', $_REQUEST[ 'page' ], 'import', $item[ 'entity_id' ], __('Import', 'wp-sukellos-fw') ),
                    'delete' => sprintf( '<a href="?page=%s&action=%s&scrapio_entity=%s">%s</a>', $_REQUEST[ 'page' ], 'delete', $item[ 'entity_id' ], __('Delete', 'wp-sukellos-fw') ),
                );
                return $this->row_actions( $actions );
            default:
                return print_r( $item, true );
        }
    }

    /**
     * Used to return the column value for cb, from DB structure retrieved from gmap_entities table
     *
     * This is method that is used to render a column when no other specific method exists for that column.
     * When WP_List_Tables attempts to render your columns (within single_row_columns()), it first checks for a column-specific method. If none exists, it defaults to this method instead.
     * This method accepts two arguments, a single $item array and the $column_name (as a slug).
     * @param $item
     * @return bool|mixed|string|void
     */
    public function column_cb( $item ) {

        return sprintf('<input type="checkbox" name="scrapio_entity[]" value="%s" />', $item[ 'entity_id' ] );
    }

    /**
     * Gets a list of sortable columns.
     *
     * The format is:
     *  'internal-name' => 'orderby'
     *  'internal-name' => array( 'orderby', bool, 'abbr', 'orderby-text', 'initially-sorted-column-order' ) –
     *  'internal-name' => array( 'orderby', 'asc' ) – The second element sets the initial sorting order.
     *  'internal-name' => array( 'orderby', true ) – The second element makes the initial order descending.
     *
     * In the second format, passing true as second parameter will make the initial sorting order be descending.
     * Following parameters add a short column name to be used as ‘abbr’ attribute, a translatable string for the current sorting, and the initial order for the initial sorted column, ‘asc’ or ‘desc’ (default: false).
     *
     * See https://developer.wordpress.org/reference/classes/wp_list_table/get_sortable_columns/
     *
     * @return array[]
     */
    public function get_sortable_columns() {

        $sortable_columns = array(
            'name' => array( 'name', true ),
            'type' => array( 'type', false ),
            'place_id' => array( 'place_id', false ),
            'zip' => array( 'zip', false ),
            'city' => array( 'city', false ),
        );
        return $sortable_columns;
    }

    /**
     * Retrieves the list of bulk actions available for this table.
     *
     * The format is an associative array where each element represents either a top level option value and label, or an array representing an optgroup and its options.
     * For a standard option, the array element key is the field value and the array element value is the field label.
     * For an optgroup, the array element key is the label and the array element value is an associative array of options as above.
     *
     * See https://developer.wordpress.org/reference/classes/wp_list_table/get_bulk_actions/
     * @return string[]
     */
    public function get_bulk_actions() {

        $actions = array(
            'delete' => __('Delete', 'wp-sukellos-fw'),
            'import' => __('Import', 'wp-sukellos-fw'),
        );
        return $actions;
    }

    /**
     * Process bulk actions : delete, import...
     * @return void
     */
    private function process_bulk_action() {

        WP_Log::debug( __METHOD__, ['$_POST'=>$_POST], 'wp-sukellos-fw' );

        if ( empty( $_POST ) || !isset( $_POST['action'] ) ) return;

        // Verify the nonce
        if (isset($_POST['_wpnonce']) && !wp_verify_nonce($_POST['_wpnonce'], 'bulk-entities')) {

            wp_die('Nonce verification failed');
        }

        // Init result message
        $this->result_message = null;

        switch( $this->current_action() ) {

            case 'delete':
                    WP_Log::debug( __METHOD__.' - Process DELETE bulk action', ['$_POST'=>$_POST], 'wp-sukellos-fw' );

                    // Handle delete action
                    $items_to_delete = isset( $_POST[ 'scrapio_entity' ] ) ? $_POST[ 'scrapio_entity' ] : array();
                    if ( !empty( $items_to_delete ) ) {

                        foreach ( $items_to_delete as $entity_id ) {

                            // Delete the item
                            WP_Scrapio_Gmap_Entities_Dao::instance()->delete_gmap_entity( $entity_id );
                        }
                    }

                    $this->result_message = __( 'All entities deleted', 'wp-sukellos-fw' );
                break;
            case 'import':
                    WP_Log::debug( __METHOD__.' - Process IMPORT bulk action', ['$_POST'=>$_POST], 'wp-sukellos-fw' );

                    // Create Wp entity CPT
                    $items_to_delete = isset( $_POST[ 'scrapio_entity' ] ) ? $_POST[ 'scrapio_entity' ] : array();
                    if ( !empty( $items_to_delete ) ) {

                        foreach ( $items_to_delete as $entity_id ) {

                            WP_Scrapio_CPT_Mapper::instance()->create_cpt_from_gmap_entity_id( $entity_id );
                        }
                    }

                    $this->result_message = __( 'All entities imported', 'wp-sukellos-fw' );
                break;
        }
    }

    /**
     * Process simple actions : delete, import...
     * @return void
     */
    private function process_simple_action() {

        WP_Log::debug( __METHOD__, ['$_GET'=>$_GET], 'wp-sukellos-fw' );

        if ( !empty( $_POST ) || empty( $_GET ) || !isset( $_GET['action'] ) ) return;

        // Init result message
        $this->result_message = null;

        switch( $this->current_action() ) {

            case 'delete':
                    WP_Log::debug( __METHOD__.' - Process DELETE simple action', ['$_GET'=>$_GET], 'wp-sukellos-fw' );
                    // Handle delete action
                    $entity_id = isset($_GET['scrapio_entity']) ? intval($_GET['scrapio_entity']) : 0;

                    // Delete the item
                    WP_Scrapio_Gmap_Entities_Dao::instance()->delete_gmap_entity( $entity_id );

                    $this->result_message = sprintf( __( 'Entity %s deleted', 'wp-sukellos-fw' ), $entity_id );
                break;
            case 'import':
                    WP_Log::debug( __METHOD__.' - Process IMPORT simple action', ['$_GET'=>$_GET], 'wp-sukellos-fw' );
                    // Handle delete action
                    $entity_id = isset($_GET['scrapio_entity']) ? intval($_GET['scrapio_entity']) : 0;

                    WP_Scrapio_CPT_Mapper::instance()->create_cpt_from_gmap_entity_id( $entity_id );

                    $this->result_message = sprintf( __( 'Entity %s imported', 'wp-sukellos-fw' ), $entity_id );
                    WP_Log::debug( __METHOD__.' - Process IMPORT simple action', ['$_GET'=>$_GET], 'wp-sukellos-fw' );
                break;
        }
    }

    /**
     * Displays extra controls between bulk actions and pagination.
     *
     * See https://developer.wordpress.org/reference/classes/wp_list_table/extra_tablenav/
     *
     * @param $which
     * @return void
     */
    public function extra_tablenav( $which ) {

        if ($which == "top") {
            ?>
            <div class="alignleft actions filtre-grille">
                <div class="colonne">
                    <label for="filter-by-place-id"><?php echo __( 'Place Id', 'wp-sukellos-fw'); ?></label>
                    <input type="text" name="place_id" id="filter-by-place-id" value="<?php echo esc_attr(isset($_REQUEST['place_id']) ? $_REQUEST['place_id'] : ''); ?>" />

                    <input type="submit" name="filter_action" id="post-query-submit" class="button" value="<?php echo __( 'Filter', 'wp-sukellos-fw'); ?>">
                </div>

                <div class="colonne">
                    <label for="filter-by-zip"><?php echo __( 'Zip', 'wp-sukellos-fw'); ?></label>
                    <input type="text" name="zip" id="filter-by-zip" value="<?php echo esc_attr(isset($_REQUEST['zip']) ? $_REQUEST['zip'] : ''); ?>" />
                </div>

                <div class="colonne">
                    <label for="filter-by-city"><?php echo __( 'City', 'wp-sukellos-fw'); ?></label>
                    <input type="text" name="city" id="filter-by-city" value="<?php echo esc_attr(isset($_REQUEST['city']) ? $_REQUEST['city'] : ''); ?>" />
                </div>
            </div>
            <?php
        }
    }

    /**
     * Prepares the list of items for displaying.
     * Developers should use this class to query and filter data, handle sorting, and pagination, and any other data-manipulation required prior to rendering.
     * This method should be called explicitly after instantiating your class, and before rendering.
     *
     * See https://developer.wordpress.org/reference/classes/wp_list_table/prepare_items/
     *
     * @return void
     */
    public function prepare_items() {

        WP_Log::debug( __METHOD__, ['REQUEST'=>$_REQUEST, 'GET'=>$_GET, 'POST'=>$_POST], 'wp-sukellos-fw' );

        //  $_column_headers
        //      In core, this property is assigned automatically. Developers must manually define it in their prepare_items() or __construct() methods.
        //      This property requires a 4-value array :
        //      - The first value is an array containing column slugs and titles (see the get_columns() method).
        //      - The second value is an array containing the values of fields to be hidden.
        //      - The third value is an array of columns that should allow sorting (see the get_sortable_columns() method).
        //      - The fourth value is a string defining which column is deemed to be the primary one, displaying the row’s actions (edit, view, etc). The value should match that of one of your column slugs in the first value.
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array( $columns, $hidden, $sortable );

        // Here to process bulk|simple actions before loading data
        $this->process_bulk_action();
        $this->process_simple_action();

        // $items
        // This is used to store the raw data you want to display. Generally you will set this property directly in the prepare_items() method.

        // Filters
        $place_id_filter = isset($_REQUEST['place_id']) ? sanitize_text_field($_REQUEST['place_id']) : null;
        $zip_filter = isset($_REQUEST['zip']) ? sanitize_text_field($_REQUEST['zip']) : null;
        $city_filter = isset($_REQUEST['city']) ? sanitize_text_field($_REQUEST['city']) : null;

        // Ordering
        $orderby = ( !empty( $_REQUEST[ 'orderby' ] ) ) ? $_REQUEST[ 'orderby' ] : 'name';
        $order = ( !empty( $_REQUEST[ 'order' ] ) ) ? $_REQUEST[ 'order' ] : 'ASC';
        $per_page = 200;
        $current_page = $this->get_pagenum();
        $offset = ( $current_page - 1 ) * $per_page;

        // Request data
        $data = WP_Scrapio_Gmap_Entities_Dao::instance()->get_gmap_entities( $place_id_filter, null, null, $zip_filter, $city_filter, $orderby, $order, $offset, null );
        $total_items = count( $data );

        $data  = array_slice($data, (($current_page-1) * $per_page), $per_page);


        // $_pagination_args
        //      Stores information needed for handling table pagination (as an array). This generally isn’t manipulated directly, but rather used with get_pagination_arg( string ) or set_pagination_args(array).
        $this->set_pagination_args( array(
            'total_items' => $total_items,
            'per_page' => $per_page,
            'total_pages' => ceil( $total_items / $per_page )
        ) );

        $this->items = $data;
    }
}

