<?php

namespace Sukellos\Dao;

use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\Utils\WP_Log;

defined( 'ABSPATH' ) or exit;

/**
 * This class is used to CRUD queries into DB
 *
 * @since 1.0.0
 */
class WP_Scrapio_Gmap_Entities_Dao {

    // Use Trait Singleton
    use Singleton;

    /**
     * Get an entity using Google place id
     * @return array|object|\stdClass[]|null
     */
    public function get_gmap_entity_from_place_id( $place_id ) {

        global $wpdb;
        $table_gmap_entities = $wpdb->prefix."scrapio_gmap_entities";

        $params = array( $place_id );

        // Build SQL
        $sql = "SELECT * FROM ".$table_gmap_entities." WHERE place_id = %s";

        $prepared_statement = $wpdb->prepare($sql, $params);
        $results = $wpdb->get_results( $prepared_statement, 'ARRAY_A' );
        WP_Log::debug(__METHOD__, ['Result of select'=>sizeof($results), 'Statement'=>$prepared_statement], 'wp-sukellos-fw');

        if ( empty( $results ) ) return null;
        return $results[0];
    }

    /**
     * Get an entity
     * @return array|object|\stdClass[]|null
     */
    public function get_gmap_entity( $entity_id ) {

        global $wpdb;
        $table_gmap_entities = $wpdb->prefix."scrapio_gmap_entities";

        $params = array( $entity_id );

        // Build SQL
        $sql = "SELECT * FROM ".$table_gmap_entities." WHERE entity_id = %d";

        $prepared_statement = $wpdb->prepare($sql, $params);
        $results = $wpdb->get_results( $prepared_statement, 'ARRAY_A' );
        WP_Log::debug(__METHOD__, ['Result of select'=>sizeof($results), 'Statement'=>$prepared_statement], 'wp-sukellos-fw');

        if ( empty( $results ) ) return null;
        return $results[0];
    }

    /**
     * Get querys
     * @return array|object|\stdClass[]|null
     */
    public function get_gmap_entities(  $place_id_filter=null, $admin1_filter=null, $admin2_filter=null, $zip_filter=null, $city_filter=null, $orderby='ID', $order='ASC', $offset=0, $limit=50 ) {

        global $wpdb;
        $table_gmap_entities = $wpdb->prefix."scrapio_gmap_entities";

        $params = array();

        // Build SQL
        $sql = "SELECT * FROM ".$table_gmap_entities." WHERE 1=1";

        // Add filters
        if ( !is_null( $place_id_filter ) ) {

            $sql .= " AND place_id LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $place_id_filter ) . '%';
        }
        if ( !is_null( $admin1_filter ) ) {

            $sql .= " AND admin1 LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $admin1_filter ) . '%';
        }
        if ( !is_null( $admin2_filter ) ) {

            $sql .= " AND admin2 LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $admin2_filter ) . '%';
        }
        if ( !is_null( $zip_filter ) ) {

            $sql .= " AND zip LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $zip_filter ) . '%';
        }
        if ( !is_null( $city_filter ) ) {

            $sql .= " AND city LIKE %s";
            $params[] = '%' . $wpdb->esc_like( $city_filter ) . '%';
        }

        $sql .= " ORDER BY ".$orderby." ".($order=='desc'?"DESC":"ASC");

        if ( !is_null( $limit ) ) {

            $params[]=$offset;
            $params[]=$limit;
            $sql .= ' LIMIT %d, %d';
        }

        $prepared_statement = $wpdb->prepare($sql, $params);
        $results = $wpdb->get_results( $prepared_statement, 'ARRAY_A' );
        WP_Log::debug(__METHOD__, ['Result of select'=>sizeof($results), 'Statement'=>$prepared_statement], 'wp-sukellos-fw');

        return $results;
    }

    /**
     * Get querys
     * @return array|object|\stdClass[]|null
     */
    public function delete_gmap_entity(  $entity_id ) {

        global $wpdb;
        $table_gmap_entities = $wpdb->prefix."scrapio_gmap_entities";

        $params = array( $entity_id );

        // Build SQL
        $sql = "DELETE FROM ".$table_gmap_entities." WHERE entity_id = %d";

        $prepared_statement = $wpdb->prepare($sql, $params);
        $results = $wpdb->get_results( $prepared_statement, 'ARRAY_A' );
        WP_Log::debug(__METHOD__, ['Result of select'=>sizeof($results), 'Statement'=>$prepared_statement], 'wp-sukellos-fw');

        return $results;
    }

    /**
     * Update an GMAP entity
     * @return array|object|\stdClass[]|null
     */
    public function update_gmap_entity(  $entity_id, $query_id=null, $google_id=null, $place_id=null, $name=null, $type=null, $country_code=null, $admin1=null, $admin2=null, $zip=null, $city=null, $entity_json=null ) {

        global $wpdb;
        $table_gmap_entities = $wpdb->prefix."scrapio_gmap_entities";

        // Vérification que l'id' est bien fournie
        if ( empty( $entity_id ) ) {

            WP_Log::debug( __METHOD__.' - $entity_id manquante', [], 'wp-sukellos-fw' );
            return false;
        }

        $params = array( $entity_id );

        // Paramètres supplémentaires avec types de données
        $params = [
            'query_id' => '%d',
            'google_id' => '%s',
            'place_id' => '%s',
            'name' => '%s',
            'type' => '%s',
            'country_code' => '%s',
            'admin1' => '%s',
            'admin2' => '%s',
            'zip' => '%s',
            'city' => '%s',
            'entity_json' => '%s',
        ];

        // Initialisation des parties dynamiques de la requête
        $columns = array();
        $params_to_bind = array();

        // Construction dynamique des colonnes et valeurs
        foreach ( $params as $column => $param_type ) {

            // Utilisation de la variable variable pour accéder aux paramètres dynamiques
            if ( !is_null( $$column ) ) {

                $columns[] = "$column = $param_type";
                $params_to_bind[] = $$column; // Ajout de la valeur correspondante
            }
        }

        // Si aucun champ à mettre à jour n'est fourni
        if (empty($columns)) {
            WP_Log::debug(__METHOD__ . ' - Aucun champ à mettre à jour', [], 'wp-sukellos-fw');
            return false;
        }

        // Ajout de la clé pour la clause WHERE
        $params_to_bind[] = $entity_id;
        $update_sql = "UPDATE $table_gmap_entities SET " . implode(', ', $columns) . " WHERE entity_id = %d";

        // Préparation de la requête SQL
        $prepared_query = $wpdb->prepare($update_sql, $params_to_bind);
        if ($prepared_query === false) {
            WP_Log::debug(__METHOD__ . ' - Prepare failed', [], 'wp-sukellos-fw');
            return false;
        }

        // Log des détails de la requête
        WP_Log::debug(__METHOD__, ['sql' => $update_sql, 'params' => $params_to_bind, 'prepared_query' => $prepared_query], 'wp-sukellos-fw');

        // Exécution de la requête
        $result = $wpdb->query($prepared_query);
        WP_Log::debug(__METHOD__, ['query result' => $result], 'wp-sukellos-fw');

        return $result;
    }
}