<?php

namespace Sukellos;

use Sukellos\WPFw\Utils\WP_Log;

defined( 'ABSPATH' ) or exit;

/**
 * Field Class - Abstract parent class for all fields
 *
 * Must be ovveride to create specific fields or specific items
 *
 * @since 1.0.0
 */
abstract class Field {

    const GMAP_ENTITY = 'scrapio';

    /**
     * Default data from Scrap.io
     * @var array
     */
    protected $data = null;

    /**
     * Field definition used for ACF
     * @var null
     */
    protected $acf_field_definition = null;

    /**
     * Name of the field
     * @var null
     */
    protected $field_id = null;

    /**
     * Label of the field
     * @var null
     */
    protected $field_label = '';

    /**
     * Constructor
     *
     * @param $settings
     */
    public function __construct() {

        $default_acf_field_definition = array(

            /* (string) Unique identifier for the field. Must begin with 'field_' */
            'key' => $this->get_acf_field_key(),

            /* (string) Visible when editing the field value */
            'label' => $this->get_acf_field_label(),

            /* (string) Used to save and load data. Single word, no spaces. Underscores and dashes allowed */
            'name' => $this->get_acf_field_name(),

            /* (string) Type of field (text, textarea, image, etc) */
            'type' => 'text',

            /* (string) Instructions for authors. Shown when submitting data */
            'instructions' => '',

            /* (int) Whether or not the field value is required. Defaults to 0 */
            'required' => 0,

            /* (mixed) Conditionally hide or show this field based on other field's values.
            Best to use the ACF UI and export to understand the array structure. Defaults to 0 */
            'conditional_logic' => 0,

            /* (array) An array of attributes given to the field element */
            'wrapper' => array(
                'width' => '',
                'class' => '',
                'id' => '',
            ),

            /* (mixed) A default value used by ACF if no value has yet been saved */
            'default_value' => '',

            'aria-label' => '',
            'maxlength' => '',
            'placeholder' => '',
            'prepend' => '',
            'append' => '',
        );
        $specific_acf_field_definition = $this->get_specific_acf_field_definition();

        $this->acf_field_definition = array_merge( $default_acf_field_definition, $specific_acf_field_definition );
    }


    /**
     * Update the data
     *
     * MUST be called before insert_custom_field()
     *
     * @param $settings
     */
    public function set_data( $data ) {

        $this->data = $data;
    }

    /**
     * ACF label of the field
     * @return string
     */
    public function get_acf_field_label() {

        return __( $this->field_label, 'wp-sukellos-fw' );
    }

    /**
     * ACF name of the field
     * @return string
     */
    public function get_acf_field_name() {

        return Field::GMAP_ENTITY.'_'.$this->field_id;
    }

    /**
     * ACF key of the field (field_...)
     * @return string
     */
    public function get_acf_field_key() {

        return 'field_'.Field::GMAP_ENTITY.'_'.$this->field_id;
    }

    /**
     * Get the field definition used to create ACF field
     */
    public function get_acf_field_definition() {

        return $this->acf_field_definition;
    }

    /**
     * Insert the field as a WordPress ACF Custom Field
     *
     * MUST be called after set_data()
     *
     * MUST be overriden if the type is complex (other than simple text, url...)
     */
    public function update_acf_field( $post_id ) {

        if ( is_null( $this->data ) ) return false;

        // Get its self data from gmap entity JSON data
        if ( !isset( $this->data[$this->field_id] ) || is_null( $this->data[$this->field_id] ) || empty( $this->data[$this->field_id] )  )  return false;

        $self_data = $this->data[$this->field_id];
        WP_Log::debug( __METHOD__, ['field_id'=>$this->field_id, '$self_data'=>$self_data], 'wp-sukellos-fw' );

        update_field( $this->get_acf_field_name(), $self_data, $post_id );
    }

    /**
     * Get the specific ACF Custom Field definition
     */
    abstract protected function get_specific_acf_field_definition();
}
