<?php

namespace Sukellos;

use Sukellos\WPFw\Utils\WP_Helper;
use Sukellos\WPFw\Utils\WP_Log;
use Exception;

defined( 'ABSPATH' ) or exit;

/**
 * Photos field Class
 *
 * @since 1.0.0
 */
class Photos extends Field {

    // "photos": {
    //		"0": "https://lh5.googleusercontent.com/p/AF1QipMglSk-OE_MbtN9P3HyKtwkpXLpktqdr8tBc6ar",
    //		"2": "https://lh5.googleusercontent.com/p/AF1QipNqC6HbsAKo1lvgsqKQ0HXtea39H_HjdNbjwpVm"
    //	},

    protected $field_id = 'photos';
    protected $field_label = 'Photos';

    /**
     * Get the specific ACF Custom Field definition
     */
    public function get_specific_acf_field_definition() {

        return array(
            'type' => 'gallery',
            'instructions' => '',
            'required' => 0,
            'return_format' => 'url',
            'library' => 'uploadedTo',
            'min' => '',
            'max' => '',
            'min_width' => '',
            'min_height' => '',
            'min_size' => '',
            'max_width' => '',
            'max_height' => '',
            'max_size' => '',
            'mime_types' => '',
            'insert' => 'append',
            'preview_size' => 'medium',
        );
    }

    /**
     * Check if image url is already present in media, using self post_meta saved on download
     *
     * @param $image_url
     * @return false|int
     */
    private function check_image_in_media_library( $image_url ) {
        global $wpdb;

        // Query the database for attachments with the same URL meta value
        $attachment_id = $wpdb->get_var( $wpdb->prepare( "
        SELECT post_id FROM {$wpdb->postmeta}
        WHERE meta_key = '_source_url'
        AND meta_value = %s
        LIMIT 1
        ", $image_url ) );

        return $attachment_id ? intval( $attachment_id ) : false;
    }

    /**
     * Download image, using raw method because of specific format of Google image URL (without suffix)
     * 
     * @param $image_url
     * @return false|int|\WP_Error
     */
    private function import_image( $image_url ) {

        // Fetch the image URL and handle redirections
        $response = wp_remote_get( $image_url, array( 'timeout' => 20 ) );
        WP_Log::debug( __METHOD__, [ '$image_url' => $image_url, '$response' => $response ], 'wp-sukellos-fw' );

        // Check for HTTP errors
        if ( is_wp_error( $response ) ) {
            return false;
        }

        // Get the content type
        $content_type = wp_remote_retrieve_header( $response, 'content-type' );
        WP_Log::debug( __METHOD__, [ '$image_url' => $image_url, '$content_type' => $content_type ], 'wp-sukellos-fw' );

        if ( strpos( $content_type, 'image' ) === false ) {
            return false;
        }

        // Get the body of the response
        $image_data = wp_remote_retrieve_body( $response );

        // Get the filename from the URL or generate a unique name
        $image_name = basename( parse_url( $image_url, PHP_URL_PATH ) );
        WP_Log::debug( __METHOD__, [ '$image_url' => $image_url, '$image_name' => $image_name ], 'wp-sukellos-fw' );

        $exts = array(".png", ".jpg", ".gif", ".jpeg", ".bmp", ".webp", ".svg", ".ico");
        $add_ext = true;
        foreach ( $exts as $ext ) {

            if ( FALSE !== strpos( $image_name, $ext ) ) {
                $add_ext = false;
                break;
            }
        }
        if ( $add_ext ) {

            switch ($content_type) {
                case 'image/jpeg':
                    $image_name .= '-'.time().'.jpg';
                    break;
                case 'image/png':
                    $image_name .= '-'.time().'.png';
                    break;
                case 'image/gif':
                    $image_name .= '-'.time().'.gif';
                    break;
                case 'image/bmp':
                    $image_name .= '-'.time().'.bmp';
                    break;
                case 'image/webp':
                    $image_name .= '-'.time().'.webp';
                    break;
                case 'image/svg+xml':
                    $image_name .= '-'.time().'.svg';
                case 'image/x-icon':
                    break;
                    $image_name .= '-'.time().'.ico';
                    break;
            }

            WP_Log::debug( __METHOD__, [ '$image_url' => $image_url, '$image_name' => $image_name ], 'wp-sukellos-fw' );
        }

        // Save the image to a temporary file
        $upload_dir = wp_upload_dir();
        $file_path = $upload_dir[ 'path' ].'/'.$image_name;
        WP_Log::debug( __METHOD__, [ '$image_url' => $image_url, '$file_path' => $file_path ], 'wp-sukellos-fw' );

        // Save the image file
        file_put_contents( $file_path, $image_data );

        // Check the file type and extension
        $file_type = wp_check_filetype( $file_path );
        WP_Log::debug( __METHOD__, [ '$image_url' => $image_url, '$file_type' => $file_type ], 'wp-sukellos-fw' );

        if ( !$file_type[ 'ext' ] ) {
            @unlink( $file_path );
            return false;
        }

        // Prepare an array of file information
        $file_array = array(
            'name' => $image_name,
            'tmp_name' => $file_path,
        );
        WP_Log::debug( __METHOD__, [ '$image_url' => $image_url, '$file_array' => $file_array ], 'wp-sukellos-fw' );

        // Handle sideload and insert into the media library
        $attachment_id = media_handle_sideload( $file_array, 0 );

        // Handle sideload errors
        if ( is_wp_error( $attachment_id ) ) {

            WP_Log::warning( __METHOD__, [ '$image_url' => $image_url, '$attachment_id' => $attachment_id ], 'wp-sukellos-fw' );
            @unlink( $file_path );
            return false;
        }

        // Add URL as meta data to the attachment
        update_post_meta($attachment_id, '_source_url', $image_url);

        return $attachment_id;
    }

    /**
     * Insert the field as a WordPress ACF Custom Field
     *
     * MUST be called after set_data()
     *
     * MUST be overriden if the type is complex (other than simple text, url...)
     */
    public function update_acf_field( $post_id ) {

        if ( is_null( $this->data ) ) return false;

        // Get its self data from gmap entity JSON data
        if ( !isset( $this->data[ $this->field_id ] ) || is_null( $this->data[ $this->field_id ] ) || empty( $this->data[ $this->field_id ] ) ) return false;

        $self_data = $this->data[ $this->field_id ];
        WP_Log::debug( __METHOD__, [ 'field_id' => $this->field_id, '$self_data' => $self_data ], 'wp-sukellos-fw' );

        // Include the WP media library functions
        if ( !function_exists( 'media_sideload_image' ) ) {

            require_once( ABSPATH.'wp-admin/includes/media.php' );
            require_once( ABSPATH.'wp-admin/includes/file.php' );
            require_once( ABSPATH.'wp-admin/includes/image.php' );
        }

        // Get all images
        $acf_galery_field = array();
        foreach ( $self_data as $img_url ) {

            // Check if the image URL already exists in the media library
            $attachment_id = $this->check_image_in_media_library( $img_url );
            if ( !$attachment_id ) {

                // Download and insert the image into the media library
                $attachment_id = $this->import_image( $img_url );
                WP_Log::debug( __METHOD__, [ '$img_url' => $img_url, '$self_data' => $self_data ], 'wp-sukellos-fw' );
            }

            if ( $attachment_id ) {

                $acf_galery_field[] = $attachment_id;
            }
        }
        WP_Log::debug( __METHOD__, [ 'field name' => $this->get_acf_field_name(), '$acf_galery_field' => $acf_galery_field ], 'wp-sukellos-fw' );

        update_field( $this->get_acf_field_name(), $acf_galery_field, $post_id );

        // Specific case: attach the first image as featured image
        if ( !empty( $acf_galery_field ) ) {

            $first_img = $acf_galery_field[0];
            set_post_thumbnail( $post_id, $first_img );
        }


    }
}
