<?php

namespace Sukellos\ScrapioAPI;

use Sukellos\WP_Sukellos_Scrapio;
use Sukellos\WP_Sukellos_Scrapio_Loader;
use Sukellos\WPFw\AdminBuilder\Admin_Builder;
use Sukellos\WPFw\AdminBuilder\Item_Type;
use Sukellos\WPFw\Api\WP_API_Base;
use Sukellos\WPFw\Singleton;
use Sukellos\WPFw\Utils\WP_Log;
use Exception;
use InvalidArgumentException;

defined( 'ABSPATH' ) or exit;

/**
 * API for processing LEO2Click requests.
 *
 * @since 1.0.0
 */
class WP_Scrapio_API extends WP_API_Base {

    // Use Trait Singleton
    use Singleton;

    /** @var bool whether API is enabled */
    private $enabled = false;

    private $scrapio_key = null;

    /**
     * Default init method called when instance created
     * This method can be overridden if needed.
     *
     * @since 1.0.0
     * @access protected
     */
    public function init() {

        $this->response_handler = 'WP_Scrapio_API_Response';
        $this->enabled          = true;

        if (!is_null($this->request_uri)) return;

        $this->request_uri = Admin_builder::get_option( WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_scrapio_endpoint_url', Item_Type::TEXT );
        $this->scrapio_key = Admin_builder::get_option( WP_Sukellos_Scrapio_Loader::instance()->get_options_suffix_param().'_scrapio_key', Item_Type::TEXT  );
        
        // Debug
        add_filter( 'wp_http_request_args', array( $this, 'filter_wp_http_request_args' ), 10, 2 );
        add_filter( 'wp_api_request_uri', array( $this, 'filter_wp_api_request_uri' ), 10, 2 );

        WP_Log::info( 'LEO2Click init API ', [ 'request_uri' => $this->request_uri, 'scrapio_key' => $this->scrapio_key ], 'wp-sukellos-fw' );
    }


    /**
     * Request arguments.
     *
     * Allow other actors to filter the request arguments. Note that
     * child classes can override this method, which means this filter may
     * not be invoked, or may be invoked prior to the overridden method
     *
     * @since 1.0.0
     * @param array $args request arguments
     * @param WP_API_Base class instance
     */
    public function filter_wp_http_request_args( $args, $api ) {

        WP_Log::debug( __METHOD__, ['$args'=>$args], 'wp-sukellos-fw' );
        return $args;
    }

    /**
     * Request URI Filter.
     *
     * Allow actors to filter the request URI. Note that child classes can override
     * this method, which means this filter may be invoked prior to the overridden
     * method.
     *
     * @since 4.1.0
     *
     * @param string $uri current request URI
     * @param WP_API_Base class instance
     */
    public function filter_wp_api_request_uri( $uri, $api ) {

        WP_Log::debug( __METHOD__, ['$uri'=>$uri], 'wp-sukellos-fw' );
        return $uri;
    }

    /**
     * Get the plugin instance (implements parent method).
     *
     * @since 1.0.0
     *
     * @return WP_Sukellos_Scrapio The extension main instance
     */
    protected function get_plugin() {

        return WP_Sukellos_Scrapio::instance();
    }


    /**
     * Check whether Leo2Click API is enabled.
     *
     * @since 1.0.0
     *
     * @return bool
     */
    protected function is_enabled() {
        return $this->enabled;
    }


    /**
     * Return a new request object
     *
     * Child classes must implement this to return an object that implements
     * \WP_API_Request which should be used in the child class API methods
     * to build the request. The returned WP_API_Request should be passed
     * to self::perform_request() by your concrete API methods
     *
     * @since 1.0.0
     *
     * @param array $args optional request arguments
     */
    protected function get_new_request( $args = array() ) {
        // Not adapted for multi parameters
    }
    
    /**
     * subscription
     * 
     * Request GET https://scrap.io/api/v1/subscription
     * 
     * @param type $raw true to retrieve raw data, otherwise false to decorate with specific class
     * @link https://apidoc.scrap.io/#subscription
     *
     * @since 1.0.0
     */
    public function subscription( $raw = false ) {
        
        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();
            
        try {
            
            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_SUBSCRIPTION );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );
            
            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->subscription( $raw );

        } catch ( Exception $e ) {
            
            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }

    /**
     * Request GET https://scrap.io/api/v1/gmap/types
     *
     * This endpoint allows you to search a Gmap type and get its id. Then you can use it in Gmap search endpoint.
     * You can search in any language (en and fr for now).
     *
     * @param string $search_term Term to search for, Not required
     * @param string $locale Search locale, Default 'en', Not required
     * @param bool $raw Boolean
     * @return type|null
     * @throws Exception
     * @link https://apidoc.scrap.io/#types
     */
    public function gmap_types( $search_term=null, $locale='en', $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {
            // Build params
            $get_params = array(
                'locale' => $locale,
            );
            if ( !is_null( $search_term ) && !empty( $search_term ) ) $get_params['search_term'] = $search_term;

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_GMAP_TYPES, $get_params );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->gmap_types( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }

    /**
     * Request GET https://scrap.io/api/v1/gmap/locations
     *
     * This endpoint allows you to search for locations (admin1, admin2 or city) and retrieve its ids.
     * You can list all the admin1 areas in one country, all the admin2 areas in one admin1 area, search a city inside a country, etc.
     * Then, you can use the ID in the gmap search endpoint.
     *
     * @param string $country_code ISO Country code (FR,US, etc.), Required
     * @param string $type Type of entity to search for (admin1, admin2, city), Required
     * @param string $admin1_code Admin 1 code (if you want to restrict to a specific admin 1 division), Not required
     * @param string $admin2_code Admin 2 code (if you want to restrict to a specific admin 2 division), Not required
     * @param string $search_term Term to search for, Not required
     * @param bool $raw Boolean to get a raw response
     * @return type|null
     * @throws Exception
     * @link https://apidoc.scrap.io/#locations
     */
    public function gmap_locations( $country_code, $type, $admin1_code=null, $admin2_code=null, $search_term=null, $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {
            // Build params
            $get_params = array(
                'country_code' => $country_code,
                'type' => $type,
            );
            if ( !is_null( $admin1_code ) && !empty( $admin1_code ) ) $get_params['admin1_code'] = $admin1_code;
            if ( !is_null( $admin2_code ) && !empty( $admin2_code ) ) $get_params['admin2_code'] = $admin2_code;
            if ( !is_null( $search_term ) && !empty( $search_term ) ) $get_params['search_term'] = $search_term;

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_GMAP_LOCATIONS, $get_params );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->gmap_locations( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }

    /**
     * Request GET https://scrap.io/api/v1/gmap/place
     *
     * This endpoint allows you to get all data related to a google place (by google_id or place_id).
     *
     * @param string $google_id Google id (ex: 0xabc:0xdef), Not required
     * @param string $place_id Place id (ex: ChIabcDeFGhIJkLMnoPqR), Not required
     * @param int $skip_data Boolean (0 = false or 1 = true) to indicate that you want to skip the data part of the response. When the data is skipped, it won't deduct any export credit, Not required
     * @param bool $raw Boolean to get a raw response
     * @return type|null
     * @throws Exception
     * @link https://apidoc.scrap.io/#place
     */
    public function gmap_place( $google_id=null, $place_id=null, $skip_data=null, $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {
            // Build params
            $get_params = array();
            if ( !is_null( $google_id ) && !empty( $google_id ) ) $get_params['google_id'] = $google_id;
            if ( !is_null( $place_id ) && !empty( $place_id ) ) $get_params['place_id'] = $place_id;
            if ( !is_null( $skip_data ) && !empty( $skip_data ) ) $get_params['skip_data'] = $skip_data;

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_GMAP_PLACE, $get_params );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->gmap_place( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }


    /**
     * Request GET https://scrap.io/api/v1/gmap/search
     *
     * With this endpoint, you can search all the google places of a specific category in a specific location, and all their details.
     *
     * We also provide powerful filters that allow you to fine-tune the search according to your needs, and get exactly the results you want.
     *
     * @param array $args list of params of type Const Gmap_Search_Param
     * @return type|null
     * @throws Exception
     * @link https://apidoc.scrap.io/#search
     */
    public function gmap_search( array $args, $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {
            // Validate params
            WP_Scrapio_API_Gmap_Search_Param::validate_params( $args );

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_GMAP_SEARCH, $args );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->gmap_search( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }

    /**
     * Request GET https://scrap.io/api/v1/gmap/enrich
     *
     * Our API includes a powerful feature that allows you to enrich a domain, URL, email, or phone number with data from Google Maps.
     *
     * With this endpoint, you can retrieve the google places related to the data provided (domain, url, email, phone), and all their details.
     *
     * @param string $per_page Number of results per page (1, 10, 25 or 50), Not required
     * @param int $skip_data Boolean (0 = false / 1 = true) to indicate whether to skip the data part of the response. When the data is skipped, it won't deduct any export credit., Not required
     * @param int $cursor Cursor pagination
     * @param string $url Exact URL mentioned on Google Maps
     * @param string $domain Domain mentioned in Google Maps
     * @param string $email Email found on the website of Google Place
     * @param string $phone Phone in international format associated with the Google Place (e.g.: 33139732419)
     * @param bool $raw
     * @return type|null
     * @throws Exception
     * @link https://apidoc.scrap.io/#enrich
     */
    public function gmap_enrich( $per_page=null, $skip_data=null, $cursor=null, $url=null, $domain=null, $email=null, $phone=null, $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {
            // Build params
            $get_params = array();
            if ( !is_null( $per_page ) && !empty( $per_page ) ) $get_params['per_page'] = $per_page;
            if ( !is_null( $skip_data ) && !empty( $skip_data ) ) $get_params['skip_data'] = $skip_data;
            if ( !is_null( $cursor ) && !empty( $cursor ) ) $get_params['cursor'] = $cursor;
            if ( !is_null( $url ) && !empty( $url ) ) $get_params['url'] = $url;
            if ( !is_null( $domain ) && !empty( $domain ) ) $get_params['domain'] = $domain;
            if ( !is_null( $email ) && !empty( $email ) ) $get_params['email'] = $email;
            if ( !is_null( $phone ) && !empty( $phone ) ) $get_params['phone'] = $phone;

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_GMAP_ENRICH, $get_params );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->gmap_enrich( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }


    /**
     * Request GET https://scrap.io/api/v1/blacklist
     *
     * This endpoint allows you to get the list of all your blacklists.
     *
     * @param type $raw true to retrieve raw data, otherwise false to decorate with specific class
     * @link https://apidoc.scrap.io/#blacklist
     *
     * @since 1.0.0
     */
    public function blacklist( $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_BLACKLIST );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->blacklist( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }


    /**
     * Request POST https://scrap.io/api/v1/blacklist/{list-name}
     *
     * This endpoint allows you to add data to a blacklist (google_id, place_id, domain or email).
     *
     * @param string $list_name custom list name
     * @param string $type Type of data to add to the blacklist (google_id,place_id,domain,email), Required
     * @param string $data Data to add to the blacklist, Required
     * @param type $raw true to retrieve raw data, otherwise false to decorate with specific class
     * @link https://apidoc.scrap.io/#add
     *
     * @since 1.0.0
     */
    public function blacklist_add( $list_name, $type='google_id', $data, $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {
            // Build params
            $post_params = array();
            if ( !is_null( $type ) && !empty( $type ) ) $post_params['type'] = $type;
            if ( !is_null( $data ) && !empty( $data ) ) $post_params['data'] = $data;

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_BLACKLIST_ADD.'/'.$list_name, array(), $post_params, 'POST' );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->blacklist_add( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }


    /**
     * Request DELETE https://scrap.io/api/v1/blacklist/{list-name}
     *
     * This endpoint allows you to delete a list.
     *
     * @param string $list_name custom list name
     * @param type $raw true to retrieve raw data, otherwise false to decorate with specific class
     * @link https://apidoc.scrap.io/#delete
     *
     * @since 1.0.0
     */
    public function blacklist_delete( $list_name, $raw = false ) {

        if (!$this->is_enabled()) {
            return null;
        }
        // Init
        $this->init();

        try {
            // Build params
            $post_params = array();

            // Build request
            $request = new WP_Scrapio_API_Request( WP_Scrapio_API_Request::REQUEST_BLACKLIST_DELETE.'/'.$list_name, array(), array(), 'DELETE' );

            // Add API key as header
            $this->set_request_header( 'Authorization', 'Bearer '.$this->scrapio_key );

            // Send request
            $response_raw = $this->perform_request( $request );

            // Get and parse response
            $response_handler = new WP_Scrapio_API_Response( $response_raw );

            return $response_handler->blacklist_delete( $raw );

        } catch ( Exception $e ) {

            WP_Log::error( __METHOD__.' : Exception ', [ 'code' => $e->getCode(), 'message' => $e->getMessage() ], 'wp-sukellos-fw' );
            throw $e;
        }
    }
}
